# -*- coding: UTF-8 -*-
__author__ = "余洋"
__doc__ = "Permission"
"""
  * @File    :   Permission.py
  * @Time    :   2024/11/06 20:27:11
  * @Author  :   余洋
  * @Version :   0.0.1
  * @Contact :   yuyangit.0515@qq.com
  * @License :   (C)Copyright 2019-2024, 希洋 (Ship of Ocean)
  * @Desc    :   
"""


class Permission:
    pass
